// Copyright 2019-2020 UPF-N4 authors. All rights reserved.
// Use of this source code is governed by a MIT-style license that can be
// found in the LICENSE file.

package message_test

import (
	"testing"

	"n4/pkg/pfcp/message"

	"go_n4/pfcp/internal/testutil"
)

func TestHeader(t *testing.T) {
	cases := []testutil.TestCase{
		{
			Description: "Normal",
			Structured: message.NewHeader(
				1, 0, 0, 1,
				50,                 // Message type
				0xffffffffffffffff, // SEID
				0xdadada,           // Sequence Number
				0,
				[]byte{ // Payload: ApplicationID IE
					0x00, 0x18, 0x00, 0x07, 0x67, 0x6f, 0x2d, 0x70, 0x66, 0x63, 0x70,
				},
			),
			Serialized: []byte{
				0x21, 0x32, 0x00, 0x17, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
				0xda, 0xda, 0xda, 0x00,
				0x00, 0x18, 0x00, 0x07, 0x67, 0x6f, 0x2d, 0x70, 0x66, 0x63, 0x70,
			},
		},
	}

	testutil.Run(t, cases, func(b []byte) (testutil.Serializable, error) {
		v, err := message.ParseHeader(b)
		if err != nil {
			return nil, err
		}
		return v, nil
	})
}
