// Copyright 2019-2020 UPF-N4 authors. All rights reserved.
// Use of this source code is governed by a MIT-style license that can be
// found in the LICENSE file.

package message_test

import (
	"go_n4/pfcp/internal/testutil"
	"n4/pkg/pfcp/ie"
	"n4/pkg/pfcp/message"
	"net"
	"testing"
	"time"
)

func TestSessionEstablishmentRequest(t *testing.T) {

	nid := ie.NewNodeID("172.55.55.101", "", "")
	fseid := ie.NewFSEID(0xbebbebebebbebebf, net.IPv4(172, 55, 55, 101), nil, nil)

	pdrid := ie.NewPacketDetectionRuleID(1)
	precedence := ie.NewPrecedence(0)

	sourceInterface := ie.NewSourceInterface(ie.SrcInterfaceAccess)
	networkInstance := ie.NewNetworkInstance("aaa")
	fteid := ie.NewFTEID(0, nil, nil, nil)
	ueIPAddress := ie.NewUEIPAddress(0x02, "10.0.0.3", "", 0)
	trafficEndpointID := ie.NewTrafficEndpointID(0)
	sdfFilter := ie.NewSDFFilter("aa", "bb", "cc", "dd", 0)
	applicationID := ie.NewApplicationID("aa")
	epsi := ie.NewEthernetPDUSessionInformation(0)

	ethernetFilterID := ie.NewEthernetFilterID(0)
	ethernetFilterProperties := ie.NewEthernetFilterProperties(0)
	ethertype := ie.NewEthertype(0)
	macAddress := ie.NewMacAddress("00:0c:29:94:61:c9", "00:0c:29:94:61:c9", "00:0c:29:94:61:c9", "00:0c:29:94:61:c9")
	ethernetPacketFilter := ie.NewEthernetPacketFilter(ethernetFilterID, ethernetFilterProperties, macAddress, ethertype, sdfFilter)

	qfi := ie.NewQFI(0)
	framedRoute := ie.NewFramedRoute("aa")
	framedRouting := ie.NewFramedRouting(0)
	framedIPv6Route := ie.NewFramedIPv6Route("aa")
	sourceInterfaceType := ie.New3GPPInterfaceType(ie.SrcGnGpU)

	pdi := ie.NewPDI(sourceInterface, fteid, networkInstance, ueIPAddress, trafficEndpointID, sdfFilter, applicationID, epsi, ethernetPacketFilter, qfi, framedRoute, framedRouting, framedIPv6Route, sourceInterfaceType)
	outerHeaderRemoval := ie.NewOuterHeaderRemoval(0, 0)
	farid := ie.NewFARID(1)
	urrid := ie.NewURRID(1)
	qerid := ie.NewQERID(1)
	activatePredefinedRules := ie.NewActivatePredefinedRules("hello")
	activationTime := ie.NewActivationTime(time.Now())
	deactivationTime := ie.NewDeactivationTime(time.Now())
	marID := ie.NewMARID(1)
	createPDR := ie.NewCreatePDR(0, pdrid, precedence, pdi, outerHeaderRemoval, farid, urrid, qerid, activatePredefinedRules, activationTime, deactivationTime, marID)

	applyAction := ie.NewApplyAction(0x02)
	destinationInterface := ie.NewDestinationInterface(ie.DstInterfaceCore)
	networkInstance2 := ie.NewNetworkInstance("bb")
	redirectInformation := ie.NewRedirectInformation(ie.RedirectAddrIPv4, "192.168.20.11", "192.168.2.22")
	outerHeaderCreation := ie.NewOuterHeaderCreation(0x0100, 0x00000001, "192.168.20.136", "", 0, 0, 0)
	transportLevelMarking := ie.NewTransportLevelMarking(10)
	forwardingPolicy := ie.NewForwardingPolicy("11")
	headerEnrichment := ie.NewHeaderEnrichment(ie.HeaderTypeHTTP, "wu", "cc")
	linkedTrafficEndpointID := ie.NewTrafficEndpointID(1)
	proxying := ie.NewProxying(2, 2)
	destinationInterface2 := ie.NewDestinationInterface(ie.DstInterfaceCore)
	forwardingParameters := ie.NewForwardingParameters(0, destinationInterface, outerHeaderCreation, networkInstance2, redirectInformation, transportLevelMarking, forwardingPolicy, headerEnrichment, linkedTrafficEndpointID, proxying, destinationInterface2)

	duplicatingParameters := ie.NewDuplicatingParameters(destinationInterface, outerHeaderCreation, transportLevelMarking, forwardingPolicy)

	barID := ie.NewBARID(1)
	createFAR := ie.NewCreateFAR(0, farid, applyAction, forwardingParameters, duplicatingParameters, barID)

	measurementMethod := ie.NewMeasurementMethod(1, 1, 1)
	reportingTriggers := ie.NewReportingTriggers(1)
	measurementPeriod := ie.NewMeasurementPeriod(time.Duration(1))
	volumeThreshold := ie.NewVolumeThreshold(1, 1, 1)
	volumeQuota := ie.NewVolumeQuota(true, true, true, 1, 1, 1)
	eventThreshold := ie.NewEventThreshold(1)
	eventQuota := ie.NewEventQuota(1)
	timeThreshold := ie.NewTimeThreshold(1)
	timeQuota := ie.NewTimeQuota(time.Duration(1))
	quotaHoldingTIme := ie.NewQuotaHoldingTime(time.Duration(1))
	droppedDLTrafficThreshold := ie.NewDroppedDLTrafficThreshold(true, true, 1, 1)
	//quotaValidityTime := ie.NewQuotaValidityTime(1)
	monitoringTime := ie.NewMonitoringTime(time.Now())
	subsequentVolumeThreshold := ie.NewSubsequentVolumeThreshold(1, 1, 1)
	subsequentTimeThreshold := ie.NewSubsequentTimeThreshold(21)
	subsequentVolumeQuota := ie.NewSubsequentVolumeQuota(11, 11, 11)
	subsequentTimeQuota := ie.NewSubsequentTimeQuota(time.Duration(10))
	subsequentEventThreshold := ie.NewSubsequentEventThreshold(22)
	subsequentEventQuota := ie.NewSubsequentEventQuota(22)
	inactivityDetectionTime := ie.NewInactivityDetectionTime(22)
	linkedURRID := ie.NewLinkedURRID(23)
	measurementInformation := ie.NewMeasurementInformation(1)
	timeQuotaMechanism := ie.NewTimeQuotaMechanism(0, 23)

	aggregatedURRID := ie.NewAggregatedURRID(22)
	muiliplier := ie.NewMultiplier(22, 22)
	aggregatedURRs := ie.NewAggregatedURRS(aggregatedURRID, muiliplier)

	ethernetInactivityTimer := ie.NewEthernetInactivityTimer(time.Duration(10))

	createURR := ie.NewCreateURR(urrid, measurementMethod, reportingTriggers, measurementPeriod, volumeThreshold, volumeQuota, eventThreshold, eventQuota, timeThreshold,
		timeQuota, quotaHoldingTIme, droppedDLTrafficThreshold, monitoringTime, subsequentVolumeThreshold, subsequentTimeThreshold, subsequentVolumeQuota, subsequentTimeQuota, subsequentEventThreshold,
		subsequentEventQuota, inactivityDetectionTime, linkedURRID, measurementInformation, timeQuotaMechanism, aggregatedURRs, farid, ethernetInactivityTimer)

	cases := []testutil.TestCase{
		{
			Description: "Normal",
			Structured: message.NewSessionEstablishmentRequest(
				nid, fseid, createPDR, createFAR, createURR, createURR,
			),
			Serialized: []byte{
				0x20, 0x32, 0x04, 0x6f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x05, 0x00, 0xac, 0x37, 0x37,
				0x65, 0x00, 0x39, 0x00, 0x0d, 0x02, 0xbe, 0xbb, 0xeb, 0xeb, 0xeb, 0xbe, 0xbe, 0xbf, 0xac, 0x37,
				0x37, 0x65, 0x00, 0x01, 0x01, 0x21, 0x00, 0x38, 0x00, 0x02, 0x00, 0x01, 0x00, 0x1d, 0x00, 0x04,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0xd2, 0x00, 0x14, 0x00, 0x01, 0x00, 0x00, 0x15, 0x00,
				0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x16, 0x00, 0x03, 0x61, 0x61, 0x61, 0x00, 0x5d, 0x00,
				0x05, 0x02, 0x0a, 0x00, 0x00, 0x03, 0x00, 0x83, 0x00, 0x01, 0x00, 0x00, 0x17, 0x00, 0x0f, 0x0f,
				0x00, 0x00, 0x02, 0x61, 0x61, 0x62, 0x62, 0x63, 0x63, 0x00, 0x00, 0x64, 0x64, 0x00, 0x00, 0x18,
				0x00, 0x02, 0x61, 0x61, 0x00, 0x8e, 0x00, 0x01, 0x00, 0x00, 0x84, 0x00, 0x6f, 0x00, 0x8a, 0x00,
				0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x8b, 0x00, 0x01, 0x00, 0x00, 0x85, 0x00, 0x45, 0x30, 0x30,
				0x3a, 0x30, 0x63, 0x3a, 0x32, 0x39, 0x3a, 0x39, 0x34, 0x3a, 0x36, 0x31, 0x3a, 0x63, 0x39, 0x30,
				0x30, 0x3a, 0x30, 0x63, 0x3a, 0x32, 0x39, 0x3a, 0x39, 0x34, 0x3a, 0x36, 0x31, 0x3a, 0x63, 0x39,
				0x30, 0x30, 0x3a, 0x30, 0x63, 0x3a, 0x32, 0x39, 0x3a, 0x39, 0x34, 0x3a, 0x36, 0x31, 0x3a, 0x63,
				0x39, 0x30, 0x30, 0x3a, 0x30, 0x63, 0x3a, 0x32, 0x39, 0x3a, 0x39, 0x34, 0x3a, 0x36, 0x31, 0x3a,
				0x63, 0x39, 0x00, 0x00, 0x88, 0x00, 0x02, 0x00, 0x00, 0x00, 0x17, 0x00, 0x0f, 0x0f, 0x00, 0x00,
				0x02, 0x61, 0x61, 0x62, 0x62, 0x63, 0x63, 0x00, 0x00, 0x64, 0x64, 0x00, 0x00, 0x7c, 0x00, 0x01,
				0x00, 0x00, 0x99, 0x00, 0x02, 0x61, 0x61, 0x00, 0x9a, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x9b, 0x00, 0x02, 0x61, 0x61, 0x00, 0xa0, 0x00, 0x01, 0x05, 0x00, 0x5f, 0x00, 0x02, 0x00, 0x00,
				0x00, 0x6c, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x51, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01,
				0x00, 0x6d, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x6a, 0x00, 0x05, 0x68, 0x65, 0x6c, 0x6c,
				0x6f, 0x00, 0xa3, 0x00, 0x04, 0x16, 0x3a, 0x81, 0x71, 0x00, 0xa4, 0x00, 0x04, 0x16, 0x3a, 0x82,
				0x1f, 0x00, 0xaa, 0x00, 0x02, 0x00, 0x01, 0x00, 0x03, 0x00, 0x9c, 0x00, 0x6c, 0x00, 0x04, 0x00,
				0x00, 0x00, 0x01, 0x00, 0x2c, 0x00, 0x01, 0x02, 0x00, 0x04, 0x00, 0x62, 0x00, 0x2a, 0x00, 0x01,
				0x01, 0x00, 0x54, 0x00, 0x0a, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0xc0, 0xa8, 0x14, 0x88, 0x00,
				0x16, 0x00, 0x02, 0x62, 0x62, 0x00, 0x26, 0x00, 0x1e, 0x00, 0x00, 0x0d, 0x31, 0x39, 0x32, 0x2e,
				0x31, 0x36, 0x38, 0x2e, 0x32, 0x30, 0x2e, 0x31, 0x31, 0x00, 0x0c, 0x31, 0x39, 0x32, 0x2e, 0x31,
				0x36, 0x38, 0x2e, 0x32, 0x2e, 0x32, 0x32, 0x00, 0x1e, 0x00, 0x02, 0x00, 0x0a, 0x00, 0x29, 0x00,
				0x03, 0x02, 0x31, 0x31, 0x00, 0x62, 0x00, 0x07, 0x00, 0x02, 0x77, 0x75, 0x02, 0x63, 0x63, 0x00,
				0x83, 0x00, 0x01, 0x01, 0x00, 0x89, 0x00, 0x01, 0x06, 0x00, 0x2a, 0x00, 0x01, 0x01, 0x00, 0x05,
				0x00, 0x20, 0x00, 0x2a, 0x00, 0x01, 0x01, 0x00, 0x54, 0x00, 0x0a, 0x01, 0x00, 0x00, 0x00, 0x00,
				0x01, 0xc0, 0xa8, 0x14, 0x88, 0x00, 0x1e, 0x00, 0x02, 0x00, 0x0a, 0x00, 0x29, 0x00, 0x03, 0x02,
				0x31, 0x31, 0x00, 0x58, 0x00, 0x01, 0x01, 0x00, 0x06, 0x01, 0x42, 0x00, 0x51, 0x00, 0x04, 0x00,
				0x00, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x01, 0x07, 0x00, 0x25, 0x00, 0x02, 0x00, 0x01, 0x00, 0x40,
				0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x00, 0x19, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x01, 0x00, 0x49, 0x00, 0x19, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
				0x00, 0x95, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x94, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01,
				0x00, 0x20, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x4a, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x47, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x48, 0x00, 0x11, 0x03, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x21, 0x00,
				0x04, 0xe2, 0x54, 0xf4, 0x86, 0x00, 0x22, 0x00, 0x19, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x01, 0x00, 0x23, 0x00, 0x04, 0x00, 0x00, 0x00, 0x15, 0x00, 0x79, 0x00, 0x19, 0x07, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x00, 0x7a, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x97, 0x00, 0x04, 0x00, 0x00, 0x00, 0x16, 0x00, 0x96, 0x00, 0x04, 0x00, 0x00, 0x00, 0x16, 0x00,
				0x24, 0x00, 0x04, 0x00, 0x00, 0x00, 0x16, 0x00, 0x52, 0x00, 0x04, 0x00, 0x00, 0x00, 0x17, 0x00,
				0x64, 0x00, 0x01, 0x01, 0x00, 0x73, 0x00, 0x09, 0x00, 0x00, 0x00, 0x17, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x76, 0x00, 0x18, 0x00, 0x78, 0x00, 0x04, 0x00, 0x00, 0x00, 0x16, 0x00, 0x77, 0x00,
				0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x16, 0x00, 0x6c, 0x00,
				0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x92, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x01,
				0x42, 0x00, 0x51, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x01, 0x07, 0x00, 0x25,
				0x00, 0x02, 0x00, 0x01, 0x00, 0x40, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x00, 0x19,
				0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x49, 0x00, 0x19, 0x07, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x95, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x94,
				0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x20, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x4a,
				0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x47, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x48,
				0x00, 0x11, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x01, 0x00, 0x21, 0x00, 0x04, 0xe2, 0x54, 0xf4, 0x86, 0x00, 0x22, 0x00, 0x19, 0x07,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x23, 0x00, 0x04, 0x00, 0x00, 0x00, 0x15,
				0x00, 0x79, 0x00, 0x19, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x00, 0x7a, 0x00,
				0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x97, 0x00, 0x04, 0x00, 0x00, 0x00, 0x16, 0x00, 0x96, 0x00,
				0x04, 0x00, 0x00, 0x00, 0x16, 0x00, 0x24, 0x00, 0x04, 0x00, 0x00, 0x00, 0x16, 0x00, 0x52, 0x00,
				0x04, 0x00, 0x00, 0x00, 0x17, 0x00, 0x64, 0x00, 0x01, 0x01, 0x00, 0x73, 0x00, 0x09, 0x00, 0x00,
				0x00, 0x17, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x76, 0x00, 0x18, 0x00, 0x78, 0x00, 0x04, 0x00,
				0x00, 0x00, 0x16, 0x00, 0x77, 0x00, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x16, 0x00,
				0x00, 0x00, 0x16, 0x00, 0x6c, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x92, 0x00, 0x04, 0x00,
				0x00, 0x00, 0x00,
			},
		},
	}

	testutil.Run(t, cases, func(b []byte) (testutil.Serializable, error) {
		v, err := message.ParseSessionEstablishmentRequest(b)
		if err != nil {
			return nil, err
		}
		v.Payload = nil
		return v, nil
	})
}
